Attribute VB_Name = "modHideSearchBox"

' ******* COPYRIGHT NOTICE *******
' This module was written by Philipp Stiefel and published at https://codekabinett.com/page.php?Theme=10&Lang=2#hide-navigation-bar-search-box
' You are granted permission to use this module in your own application free of charge, as long as you leave this copyright notice in place.
' You may NOT publish this sample on its own without significant functional changes or enhancements without written consent of the original author!


Option Compare Database
Option Explicit
Private Type RECT
        Left As Long
        top As Long
        Right As Long
        Bottom As Long
End Type

Private Declare PtrSafe Function IsWindow Lib "user32" (ByVal hWnd As LongPtr) As Long
Private Declare PtrSafe Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As LongPtr, ByVal wMsg As Long, ByVal wParam As LongPtr, lParam As LongPtr) As Long
Private Declare PtrSafe Function FindWindowEx Lib "user32" Alias "FindWindowExA" (ByVal hWndParent As LongPtr, ByVal hWndChildAfter As LongPtr, _
                                  ByVal lpszClassName As String, ByVal lpszWindowName As String) As LongPtr

Private Declare PtrSafe Function ShowWindow Lib "user32" (ByVal hWnd As LongPtr, ByVal nCmdShow As Long) As Long

Private Declare PtrSafe Function SetClassLongPtr Lib "user32" Alias "SetClassLongA" (ByVal hWnd As LongPtr, ByVal nIndex As Long, ByVal dwNewLong As LongPtr) As LongPtr
Private Declare PtrSafe Function GetClassLongPtr Lib "user32" Alias "GetClassLongA" (ByVal hWnd As LongPtr, ByVal nIndex As Long) As LongPtr
Private Declare PtrSafe Function GetParent Lib "user32" (ByVal hWnd As LongPtr) As LongPtr
Private Declare PtrSafe Function GetWindowRect Lib "user32" (ByVal hWnd As LongPtr, ByRef lpRect As RECT) As Long
Private Declare PtrSafe Function GetDC Lib "user32" (ByVal hWnd As LongPtr) As LongPtr


Private Const GCLP_HBRBACKGROUND = (-10)
Private Const SW_HIDE = 0

Private Const WM_ERASEBKGND = &H14

Public Sub HideNavigationBarSearchBox(ByVal formWindowHandle As LongPtr)

    Dim RecNavHostWindowHandle As LongPtr
    Dim SUIWindowHandle As LongPtr
    Dim NetUIHWNDChild As LongPtr
    
    SUIWindowHandle = FindWindowEx(formWindowHandle, 0, "OSUI", "SUI")
    RecNavHostWindowHandle = FindWindowEx(SUIWindowHandle, 0, "NetUINativeHWNDHost", "RecNavHost")
    NetUIHWNDChild = FindWindowEx(RecNavHostWindowHandle, 0, "NetUIHWND", "")

    If NetUIHWNDChild <> 0 Then
        Dim foundWindow As LongPtr
        Dim foundWindows As Collection
        Set foundWindows = New Collection
        Do
            foundWindow = FindWindowEx(NetUIHWNDChild, foundWindow, "NetUICtrlNotifySink", "")
            If foundWindow <> 0 Then
                foundWindows.Add foundWindow
            End If
        Loop Until foundWindow = 0
    
        If foundWindows.Count > 0 Then
        
            Dim ParentOfSearchTextBoxWindowHandle As LongPtr
            ParentOfSearchTextBoxWindowHandle = GetRightmostWindow(foundWindows)
        
            Dim SearchTextBoxWindowHandle As LongPtr
            SearchTextBoxWindowHandle = FindWindowEx(ParentOfSearchTextBoxWindowHandle, 0, "RICHEDIT60W", "")
     
            If IsWindow(ParentOfSearchTextBoxWindowHandle) _
                And IsWindow(SearchTextBoxWindowHandle) _
            Then
                ShowWindow SearchTextBoxWindowHandle, SW_HIDE
                Dim bgBrush As LongPtr
                ' the following two code lines are a bit of a hack
                ' We highjack the background brush of the navigation button host window, because it has the correct color
                ' then we inject this as background brush into the *CLASS* (=will affect all windows of that class) of the parent window of the search box
                ' Yes, this appears to be reckless, but I think it isn't (and not just because 'it works for me'). - Prove me wrong!
                bgBrush = GetClassLongPtr(RecNavHostWindowHandle, GCLP_HBRBACKGROUND)
                ' This works on Access 2016 and newer, in older versions RecNavHostWindow has no background brush
                If bgBrush = 0 Then
                    Const COLOR_SCROLLBAR As Long = 0   ' closest system color to background color
                    bgBrush = COLOR_SCROLLBAR + 1
                End If
                    
                SetClassLongPtr ParentOfSearchTextBoxWindowHandle, GCLP_HBRBACKGROUND, bgBrush
                ' Not sure if we should delete the old brush here, I rather don't even if we leak a resource then.
                
                Dim hDc As LongPtr
                hDc = GetDC(ParentOfSearchTextBoxWindowHandle)
                SendMessage ParentOfSearchTextBoxWindowHandle, WM_ERASEBKGND, hDc, 0

            End If
        End If
    
    End If

End Sub


Private Function GetRightmostWindow(windows As Collection) As LongPtr
    Dim RightmostWindowHandle As LongPtr
    Dim leftPos As Long
    Dim windowRect As RECT
    Dim windowHandle As Variant
    For Each windowHandle In windows
        If CBool(GetWindowRect(CLngPtr(windowHandle), windowRect)) Then
            If leftPos < windowRect.Left Then
                leftPos = windowRect.Left
                RightmostWindowHandle = CLngPtr(windowHandle)
            End If
        End If
    Next
    
    GetRightmostWindow = RightmostWindowHandle
End Function

' ******* COPYRIGHT NOTICE *******
' This module was written by Philipp Stiefel and published at https://codekabinett.com/page.php?Theme=10&Lang=2#hide-navigation-bar-search-box
' You are granted permission to use this module in your own application free of charge, as long as you leave this copyright notice in place.
' You may NOT publish this sample on its own without significant functional changes or enhancements without written consent of the original author!
